/*------------------------------------------------------------------------------

READING AND WRITING TO AN EXTERNAL DEVICE WITH THE DATA BUS

This example application demonstrates how to do a simple 1-byte data bus r/w 
operation.  The external hardware that is used for this example as attached 
memory is a 74HC574 octal flip-flop with tri-state outputs.  Since the bus is
configured to have a 8-bit data path in this example we will be using the data 
bus pins DB17-DB23 from header J14.  These data bus signals are to be connected
to the corresponding inputs AND outputs of the external 74hc574 chip.  The 
NOT Output Enable(/OE) signal at J1 pin 8 is then connected to the 
NOT Output Enable pin on the 74hc574.  The /OE signal will be de-asserted only
during bus reads so clashing will not occur on the bus.  The on-board ram is 
directly connected to the data bus pins so any signal on the bus line, other 
then during an external read, will cause the processor to crash/trap.  The last
signal necesary to perform a bus operation is the NOT Chip Select 1 (/CS1)
signal located on J1 pin 5.  This signal will connect to the clock input of the
74hc574 chip and is asserted when the bus is ready to read or write to the bus.

More information on initializing the processor for data bus operations can be
found in chapters 8 and 20 on the MCF5272 user manual.
-----------------------------------------------------------------------------*/

#include "predef.h" 
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <..\MOD5272\system\sim5272.h>

#define BASEADDRESS (0x02800000 )


extern "C" { 
void UserMain(void * pd);
}

const char * AppName="BusRW72";

static volatile PBYTE ExtByte;    //Address of external memory
volatile char RAMByte;         //Data for external memory

void InitBus()
{
   sim.cs[1].csor = 0xFFFFFE40;   //compare all address and: 1110 0100 0000
   sim.cs[1].csbr = ( BASEADDRESS | 0x101 );// 0010 0000 0001
   ExtByte = ( PBYTE ) BASEADDRESS;   //Address associated with external memory byte
}

void EnterHex()
{
   printf("\r\n\r\nEnter char to store in ram: ");
   RAMByte = getchar();
   return;
}
            
             
void UserMain(void * pd)
{
 InitializeStack();
 if (EthernetIP==0)GetDHCPAddress(); 
 OSChangePrio(MAIN_PRIO);
 EnableAutoUpdate();
 iprintf("Application started\r\n\r\n");
   
 InitBus();   // Configuration of the external memory access from data bus 
 
 while (1)
  {
   iprintf("\r\n\r\nPress 1 to enter char value into ram from UART\r\n");
   iprintf("Press 2 to Output char onto UART from ram\r\n");
   iprintf("Press 3 to write ram to external bus\r\n");
   iprintf("Press 4 to read bus and store in ram\r\n");
   iprintf("Command:");
   
   char choice = getchar();
   
   if (choice == '1'){
      EnterHex();
   }
   if (choice == '2'){
      printf("\r\n\r\nThe value in ram is %c\r\n\r\n", RAMByte);
   }
   if (choice == '3'){
      *ExtByte = RAMByte;   //write to address assigned to external memory for bus write
   }
   if (choice == '4'){
      RAMByte = *ExtByte;  //read from address assigned to external memory for bus read
   }
  }
}
